<?php
class BBCode {
	//Check tags and eventually close malformed tags, return BBCoded String
	function CheckTags($str) {
		//array of known tags
		$known = array('p','b','u','i','s','center','right','img','url','quote', 'color', 'size');
		//storage stack
		$tags = array();
	
		for ($pos = 0; $pos<strlen($str); $pos++) {
			if ($str{$pos} == '[') {
				$end_pos = strpos($str, ']', $pos);
				$tag = substr($str, ++$pos, $end_pos-$pos);
				//deals with tags which contains arguments (ie quote)
				if (($equal_pos = strpos($tag, '=', 0)) !== FALSE) {
					$tag = substr($tag, 0, $equal_pos);
				}
				//check whether we have a defined tag or not.
				if (in_array(strtolower($tag),$known) || in_array(strtolower(substr($tag,1)),$known)) {
					//closing tag
					if ($tag{0} == '/') {
						//cleaned tag
						$tag = substr($tag, 1);		
						$before_tag = substr($str, 0, $pos-1);
						$after_tag = substr($str, $end_pos+1);	
						//pop stack
						while (($temp=array_pop($tags))) {
							if ($temp != $tag) {
								$before_tag.='[/'.$temp.']';
							} else {
								$before_tag.='[/'.$tag.']';
								break;
							}
						}
						$end_pos += strlen($before_tag)+strlen($after_tag)-strlen($str);
						$str = $before_tag.$after_tag;
					} else { // push stack
					array_push($tags,$tag);
					}
				}
				$pos = $end_pos;
			}
		}
		// empty stack and closing tags
		while ($temp = array_pop($tags)) {		
			$str .= '[/'.$temp.']';
		}
	
		return $str;
	}
	
	// return string, HTML version of BBCoded $comment
	function ParseComment($comment) {
		global $blog;
		include(dirname(__FILE__).'/BBCodeComs_config.php');
		$comment = BBCode::CheckTags($comment);
		$comment = htmlentities($comment, ENT_COMPAT, $blog->encoding);
		$patterns = array();
		$replacements = array();
		//Paragraph
		if ($BBCodeComs_config['p']) {
			$patterns[] = '#\[p\](.*?)\[/p\]#is';
			$replacements[] = '<p>\\1</p>';
		}
		// Bold
		if($BBCodeComs_config['b']) {
			$patterns[] = '#\[b\](.*?)\[/b\]#is';
			$replacements[] = '<strong>\\1</strong>';
		}
		//Italic
		if ($BBCodeComs_config['i']) {
			$patterns[] = '#\[i\](.*?)\[/i\]#is';
			$replacements[] = '<em>\\1</em>';
		}
		//Underline	
		if ($BBCodeComs_config['u']) {
			$patterns[] = '#\[u\](.*?)\[\/u\]#is';
			$replacements[] = '<ins>\\1</ins>';
		}
		//Strikethrough
		if ($BBCodeComs_config['s']) {
			$patterns[] = '#\[s\](.*?)\[/s\]#is';
			$replacements[] = '<del>\\1</del>';
		}
		//Center
		if ($BBCodeComs_config['center']) {
			$patterns[] = '#\[center\](.*?)\[/center\]#is';
			$replacements[] = '</p><div style="text-align: center;"><p>\\1</p></div><p>';
		}
		//Right
		if ($BBCodeComs_config['right']) {
			$patterns[] = '#\[right\](.*?)\[/right\]#is';
			$replacements[] = '</p><div style="text-align: right;"><p>\\1</p></div><p>';
		}
		//Colours
		if ($BBCodeComs_config['color']) {
			$patterns[] = "#\[color=(\#[0-9A-F]{6}|[a-z]+)\](.*?)\[/color\]#si";
			$replacements[] = '<span style="color: \\1;">\\2</span>';
		}
		//Size
		if($BBCodeComs_config['size']) {
			$patterns[] = "#\[size=([1-2]?[0-9])\](.*?)\[/size\]#si";
			$replacements[] = '<span style="font-size: \\1px; line-height: normal;">\\2</span>';
		}
		// Quotes
		if ($BBCodeComs_config['quote']) {
			$patterns[] = "#\[quote\](.*?)\[/quote\]#is";
			$replacements[] = '</p><blockquote><p style="margin: 2px; font-size: 11px; line-height: normal;">\\1</p></blockquote><p>';
			//Quotes with "user"
			$patterns[] = "#\[quote=&quot;(.*?)&quot;\](.*?)\[/quote\]#is";
			$replacements[] = '</p><blockquote><p style="margin: 2px; font-size: 11px; line-height: normal;"><strong>\\1 :</strong><br />\\2</p></blockquote><p>';
			//Quotes with user
			$patterns[] = "#\[quote=(.*?)\](.*?)\[/quote\]#is";
			$replacements[] = '</p><blockquote><p style="margin: 2px; font-size: 11px; line-height: normal;"><strong>\\1 :</strong><br />\\2</p></blockquote><p>';
		}
		//Images
		if ($BBCodeComs_config['img']) {
			$patterns[] = "#\[img\](.*?)\[/img\]#si";
			$replacements[] = "<img src=\"\\1\" alt=\"\" />";
		}
		//[url]xxxx://www.zzzz.yyy[/url]
		if ($BBCodeComs_config['url']) {
			$patterns[] = "#\[url\]([\w]+?://[^ \"\n\r\t<]*?)\[/url\]#is"; 
			$replacements[] = '<a href="\\1"'.($BBCodeComs_config['url_usetarget'] ? ' target="_blank"' : '').'>\\1</a>'; 
			//[url]www.zzzzz.yyy[/url]
			$patterns[] = "#\[url\]((www|ftp)\.[^ \"\n\r\t<]*?)\[/url\]#is"; 
			$replacements[] = '<a href="http://\\1"'.($BBCodeComs_config['url_usetarget'] ? ' target="_blank"' : '').'>\\1</a>'; 
			//[url=xxxx://www.zzzzz.yyy]ZzZzZ[/url] /*No I ain't sleeping yet*/
			$patterns[] = "#\[url=([\w]+?://[^ \"\n\r\t<]*?)\](.*?)\[/url\]#is"; 
			$replacements[] = '<a href="\\1"'.($BBCodeComs_config['url_usetarget'] ? ' target="_blank"' : '').'>\\2</a>'; 
			// [url=www.zzzzz.yyy]zZzZz[/url] /*But I'm thinking about*/
			$patterns[] = "#\[url=((www|ftp)\.[^ \"\n\r\t<]*?)\](.*?)\[/url\]#is"; 
			$replacements[] = '<a href="http://\\1"'.($BBCodeComs_config['url_usetarget'] ? ' target="_blank"' : '').'>\\3</a>'; 
			// [url="www.zzzzz.yyy"]zZzZz[/url]   /* It's nearly 2 am now */
			$patterns[] = "#\[url=&quot;((www|ftp)\.[^ \n\r\t<]*?)&quot;\](.*?)\[/url\]#is";
			$replacements[] = '<a href="http://\\1"'.($BBCodeComs_config['url_usetarget'] ? ' target="_blank"' : '').'>\\3</a>';
			//[url="http://www.zzzzz.yyy"]zZzZz[/url] /*I really dislike commenting code*/
			$patterns[] = "#\[url=&quot;([\w]+?://[^ \n\r\t<]*?)&quot;\](.*?)\[/url\]#is";
			$replacements[] = '<a href="\\1"'.($BBCodeComs_config['url_usetarget'] ? ' target="_blank"' : '').'>\\2</a>';
		}
		//[email]samvure@gmail.com[/email]
		if ($BBCodeComs_config['email']) {
			$patterns[] = "#\[email\]([a-z0-9&\-_.]+?@[\w\-]+\.([\w\-\.]+\.)?[\w]+)\[/email\]#is";
			$replacements[] = '<a href="mailto:\\1">\\1</a>';
		}
		$comment = preg_replace($patterns, $replacements, $comment);
		return $comment;
	}
	
	// Returns the enabled BBCode tags
	function getEnabledBBCode() {
		include(dirname(__FILE__).'/BBCodeComs_config.php');
		
		$text = '<p style="margin: 0 0 10px 0; color: #045097; font-size: 14px; font-family: Arial, sans-serif; font-weight: bold;">BBCodes activés :</p>'."\n".
			'<div style="margin: 0; color: #616F7D; font-size: 12px; font-family: Arial, sans-serif;">'."\n";
		$BBCodeTags = array();
		//$BBCodeTags[] = array('p', 'Paragraphe', '[p]abcdef[/p]');
		$BBCodeTags[] = array('b', 'Gras', '[b]abcdef[/b]');
		$BBCodeTags[] = array('i', 'Italique', '[i]abcdef[/i]');
		$BBCodeTags[] = array('u', 'Souligné', '[u]abcdef[/u]');
		$BBCodeTags[] = array('s', 'Barré', '[s]abcdef[/s]');
		$BBCodeTags[] = array('center', 'Centré', '[center]abcdef[/center]');
		$BBCodeTags[] = array('right', 'A droite', '[right]abcdef[/right]');
		$BBCodeTags[] = array('color', 'Couleur', '[color=#CC0000]abcdef[/color]<br />&nbsp;&nbsp;&nbsp;<span style="font-size: 10px; font-family: Arial, sans-serif; color: #333333;">Couleurs du type <em>red</em>, <em>blue</em>, etc... ou <em>#FF0033</em>, <em>#6699FF</em>, etc...</span>');
		$BBCodeTags[] = array('size', 'Taille', '[size=20]abcdef[/size]<br />&nbsp;&nbsp;&nbsp;<span style="font-size: 10px; font-family: Arial, sans-serif; color: #333333;">Tailles allant de 5 &agrave; 29</span>');
		$BBCodeTags[] = array('quote', 'Citation', '[quote=k-net]abcdef[/quote]<br />&nbsp;&nbsp;&nbsp;<span style="font-size: 10px; font-family: Arial, sans-serif; color: #333333;">[quote=k-net][/quote]&nbsp;&nbsp;ou&nbsp;&nbsp;[quote][/quote]</span>');
		$BBCodeTags[] = array('img', 'Image', '[img]http://www.site.com/image.jpg[/img]');
		$BBCodeTags[] = array('url', 'Lien', '[url=http://www.dotclear.net/]abcdef[/url]');
		$BBCodeTags[] = array('email', 'Email', '[email]abc@def.com[/email]');
		foreach ($BBCodeTags as $v) {
			if ($BBCodeComs_config[$v[0]]) {
				$text .= '<label style="display: block; float: left; margin: 0; width: 100px; font-weight: bold; line-height: 14px;">'.$v[1].'</label><div style="margin: 0; font-family: \'Courier New\', Courier, mono; color: #999999; line-height: 14px;">'.$v[2].'</div>'."\n";
			}
		}
		$text .= '</div>'."\n";
		return $text;
	}
}
?>